//This is to get the input from the vertex shader
//It must match the varyings from the vertex shader
vec4 vPosition;
vec4 vColor;
vec2 vTexcoord;

//Returns the depth from the given color
//Only uses the blue and alpha values
float getDepth(vec4 color)
{
    float bv = floor(color.b * 256);
    float av = floor(color.a * 256);
    float returnValue = (bv * 256) + av;
    return returnValue;
}

//Returns the "difference" between the two given colors
//This is where a bulk of the math is done
float difference(vec4 thisColor, vec4 otherColor)
{
	//Get the difference in the red and green values (where the normals are packed)
	float redDifference = thisColor.r - otherColor.r;
	float greenDifference = thisColor.g - otherColor.g;
	//Get the total difference between the two packed normals
	//The two represents how important the normals are to the outline (higher=more important)
	float dotDifference = abs(redDifference * greenDifference) * 2;
	//Get the depth difference
	float dif = (getDepth(thisColor)/65536) - (getDepth(otherColor)/65536);
	//Takes the difference in depth and uses saturate to remove all values below .01
	//Multiplying it by 700 ensures the value is easily above 1 before getting saturated
	dif = clamp(700*(dif-.01),0,1);
	//Drawing the outline on a face that is perpendicular to the camera is a bad idea
	//It leaves really ugly artifacts
	//So any face that is perpendicular to the camera will reduce the difference
	dif *= thisColor.r - .4;
	//Increment the total difference by the amount of dot difference
	dif += dotDifference;
	return dif;
}

//Values for one pixel (set within the draw event)
//Set it to 1 divided by the width or height
//If a surface is 1024 by 1024 then set both values to (1/1024)
uniform float oneHorizontal;
uniform float oneVertical;

void main()
{
	//The resolution for sampling neighbors. Higher is better but more expensive
	int pixelCount = 8;
	//How far away to sample. Higher is a thicker border and more expensive
	float maxDistance = 2;
	//^ Play with these values if you want ^
	//(the total number of pixel operations will be (pixelCount * maxDistance)
	
	//Get the pixel at the current position
	vec4 textureDataOriginal = texture2D(gm_BaseTexture,INPUT.vTexcoord);
	//The totalDifference measured the total difference between the pixel and its neighbors
	float totalDifference = 0;
	//The direction currently sampling
	float lookDirection = 0;
	//The amount to turn after each sample
	float turnAmount = 6.283185/pixelCount;
	
	//Loop through the number of max distances
	for (int jj=1; jj <= maxDistance; jj+=1)
	{
		//Pixels further away will have less influence on the "totalDifference"
		float distanceMultiply = (jj * maxDistance) * .6;
		//How far away the pixels will be
		float wMultiply = (oneHorizontal * distanceMultiply);
		float hMultiply = (oneHorizontal * distanceMultiply);
		
		//Perform division outside of loop so we don't have to do it in the loop
		distanceMultiply = 1/distanceMultiply;
		
		for(int ii = 0; ii < pixelCount; ii += 1)
		{
			//Sample the texture at the positions
			//Simple math to look in the lookDirection
			vec4 textureDataNew =
			texture2D(gm_BaseTexture,
				vec2(INPUT.vTexcoord.x + (cos(lookDirection) * wMultiply),
						INPUT.vTexcoord.y + (sin(lookDirection) * hMultiply)));
			//Increment totalDifference based on the difference multiplied by the distanceMultiplier
			totalDifference += difference(textureDataOriginal, textureDataNew) * distanceMultiply;
			//Turn the look direction
			lookDirection += turnAmount;
		}
	}
	
	//Use the total difference as the alpha
	//Any color outline can be used if you want to try some other effects
	gl_FragData[0] = vec4(0,0,0,totalDifference);
}