//This needs to match up to the VertexShaderOutput in the vertex shader
struct PixelShaderInput {
	float4 vPosition : SV_POSITION;
    float4 vColor    : COLOR;
    float2 vTexcoord : TEXCOORD;
	float4 vNormal : NORMAL0;
	float4 vCamera : NORMAL1;
	float vDepth : DEPTH;
};

//This is used to write to two separate render targets (surfaces)
struct PixelShaderOutput
{
	//The target to draw the texture with lighting
    float4 outDiffuse  : SV_TARGET0;
	//The target to draw the normal and depth information to
	float4 outNormal   : SV_TARGET1;
};

//This is a texture that will be used for the lighting ramp
Texture2D rampTexture : register(t1);
SamplerState ssRampTexture : register(s1);

//This is the direction for the light
uniform float3 lightDirection;

//Here's where the magic happens
PixelShaderOutput main(PixelShaderInput INPUT)
{
	//Create the output struct to return multiple colors
	PixelShaderOutput OUTPUT;
	
	//--Color
		//Dot values are changed so that they range from 0,1 instead of -1,1 
		//Get the dot value between the normal and the light direction
		float lightDot = (dot(INPUT.vNormal.rgb,lightDirection) *.5) + .5;
		//Get the dot value between the normal and the camera
		float cameraDot = (dot(INPUT.vNormal.rgb,INPUT.vCamera.rgb) *.5) + .5;
		//Get the dot value relative to the camera's up (using a swizzle)
		float upDot = (dot(INPUT.vNormal.rgb,INPUT.vCamera.rbg) *.5) + .5;
		//How far along the ramp texture to go
		//Change the .35 to determin how much impact the light has on the shading vs the camera angle
		//(0=only light has an impact and 1=only the camera has an impact)
		float rampAmount = lerp(lightDot,cameraDot,.35);
    
		//Get the actual texture
		float4 diffuseTexture = gm_BaseTextureObject.Sample(gm_BaseTexture, INPUT.vTexcoord);	
		//Get the lighting color based on the ramp amount
		float4 rampColor = rampTexture.Sample(ssRampTexture,float2(rampAmount,rampAmount));
	
		//Cheat lighting
		//Shade is based on the cameraDot. 1.4 is sort of the threshold. Higer=more shade
		float shade = saturate(cameraDot * 1.4);
		//Extra cheat lighting. Anything below .9 gets cut out.
		//Lower .9 for more extra light (the two values should always add up to 1)
		//For example: clamp((lightDot-.7),0,.3) ... clamp((lightDot-.4),0,.6)
		float extraLight = clamp((lightDot-.9),0,.1);
		//(I think adding a little bit of normal shading on top of the toon shading looks good)
		//If you don't want it then remove the "shade" and "extraLight" lines
		//And change the comments on on the "OUTPUT.outDiffuse.rgb = ..." line
	
		//The OUTPUT.outDiffuse will write to the 0th surface supplied in the draw event
		//Return the texture color multiplied by the ramp color and shade value and then adding in the extra light
		OUTPUT.outDiffuse.rgb = (diffuseTexture.rgb * rampColor.rgb * shade) + extraLight;
		//To remove cheat lighting ^Comment this out^
		//And uncomment  v this line v
		//OUTPUT.outDiffuse.rgb = (diffuseTexture.rgb * rampColor.rgb);
		
		//Set the alpha to the normal texture alpha
		OUTPUT.outDiffuse.a = diffuseTexture.a;
	//--End Color
	
	//--Normal and depth
		//The OUTPUT.outNormal will write to the 1th surface supplied in the draw event
		
		//Higher depth resolution means closer pixels will stand apart better
		//But it also caps how far away something can be before the depths become indistinguishable
		float depthResolution = 250.0;
		//Red is set based on the dot value of the camera's pointing direction and the surface normal
		OUTPUT.outNormal.r = cameraDot;
		//Green is set based on the dot value of the camera's up direction and the surface normal
		OUTPUT.outNormal.g = upDot;
		//Blue is set based on the depth of the vertex
		OUTPUT.outNormal.b = INPUT.vDepth;
		//Alpha is set based on the depth of the vertex but it loops based on the depthResolution
		OUTPUT.outNormal.a = (INPUT.vDepth * depthResolution) % 1.00;
	//--End normal and depth

	//Return the OUTPUT struct that contains both colors
	return OUTPUT;
}